import { NuGetClient, NuGetApiVersion } from './base';
/**
 * The default URL for the NuGet v3 API index.
 */
export declare const defaultIndexURLV3 = "https://api.nuget.org/v3/index.json";
/**
 * Determine whether the specified base URL represents the index for v3 of the NuGet API.
 *
 * @param indexURL The URL to examine.
 */
export declare function isV3IndexURL(indexURL: string): boolean;
/**
 * A client for v3 of the NuGet API.
 */
export declare class NuGetClientV3 extends NuGetClient {
    /**
     * The URLs for API end-points used by the client.
     */
    readonly autoCompleteApiUrls: string[];
    /**
     * Create a new NuGet v3 API client.
     *
     * @param apiResources Available NuGet API resources (used to select API end-points).
     */
    constructor(apiResources: NuGetApiResource[]);
    /**
     * The NuGet API version used by the client.
     */
    readonly apiVersion: NuGetApiVersion;
    /**
     * Provide suggestions to complete a package Id.
     *
     * @param partialId The partial package Id on which the suggestions will be based.
     * @returns {Promise<string[]>} A promise that resolves to all suggested package Ids.
     */
    suggestPackageIds(partialId: string, pageSize?: number): Promise<string[]>;
    /**
     * Get available versions for the specified package.
     *
     * @param packageId The Id of the target package.
     * @returns {Promise<string[]>} A promise that resolves to all suggested package versions.
     */
    getAvailablePackageVersions(packageId: string, pageSize?: number): Promise<string[]>;
    /**
     * Compare the specified package versions for sorting purposes.
     *
     * @param version1 The first package version.
     * @param version2 The second package version.
     * @returns 0, if the versions are equal, 1 if version1 is greater, or -1 if version2 is greater.
     */
    private comparePackageVersions(version1, version2);
    /**
     * Create a new v3 API client from the NuGet v3 API index.
     *
     * @param apiIndexURL The URL of the API index to use (defaults to defaultIndexURLV3).
     */
    static createFromIndex(apiIndexURL?: string): Promise<NuGetClientV3>;
}
/**
 * Represents the index response from the NuGet v3 API.
 */
export interface NuGetIndex {
    /**
     * Available API resources.
     */
    resources: NuGetApiResource[];
}
/**
 * Represents a NuGet API resource.
 */
export interface NuGetApiResource {
    /**
     * The resource Id (end-point URL).
     */
    '@id': string;
    /**
     * The resource type.
     */
    '@type': 'SearchQueryService' | 'SearchAutocompleteService' | 'PackageBaseAddress/3.0.0';
    /**
     * An optional comment describing the resource.
     */
    comment?: string;
}
